#include <math.h>
#include <stdarg.h>
#include <string.h>
#include "ggi_widget.h"
#include "ggi_widget_internal.h"
#include "ggi_widget_highlevel.h"
#include <ggi/keyboard.h>

#define MAXNUMLEN 16

struct optioncyclerdata {

	ggi_widget_size_t drawpos,drawsize;
	int	doopt;
	int	is_inside;
	int	currvalue;
	int	numparms;
	char  **optionlist;
	ggi_widget_t lab;
};

static int optioncycler_getsize(ggi_widget_t self) {

	ggi_widget_t curchild;
	
	self->min.x=self->min.y=0;
	self->opt.x=self->opt.y=0;
	
	if ((curchild=self->getchild(self,0))) {
		self->min=curchild->min;
		self->opt=curchild->opt;
	}
	return 0;
}

static int ggiWidgetOptioncyclerRedraw(ggi_widget_t self,int flush) {

	struct optioncyclerdata *mine=self->priv;
	int rc;

	ggiWidgetDrawBorder(self,&mine->drawpos,&mine->drawsize);
	rc=ggiWidgetRedrawChildren(self);
	if (flush)
		ggiFlushRegion(self->vis,self->pos.x,self->pos.y,self->size.x,self->size.y);
	return rc;
}

static int ggiWidgetOptioncyclerVischanged(ggi_widget_t self) {

	struct optioncyclerdata *mine=self->priv;
	ggi_widget_t curchild;
	ggiWidgetGetPlacement(self,&mine->drawpos,&mine->drawsize,&mine->doopt,1,1);

	if ((curchild=self->getchild(self,0))) {
		curchild->vis=self->vis;
		curchild->pos.x =self->pos.x+mine->drawpos.x;
		curchild->pos.y =self->pos.y+mine->drawpos.y;
		curchild->size.x=mine->drawsize.x;
		curchild->size.y=mine->drawsize.y;
		curchild->vischanged(curchild);
	}
	return 0;
}

static int ggiWidgetOptioncyclerChildchanged(ggi_widget_t self,ggi_widget_t child) {

	optioncycler_getsize(self);	/* Recalc my size */
	
	if (self->parent) self->parent->childchanged(self->parent,self);
	else              self->vischanged(self);

	return 0;
}

static void cb_new_value(ggi_widget_t self) {

	struct optioncyclerdata *mine=self->priv;

	while(mine->currvalue<0)
		mine->currvalue+=mine->numparms;
	while(mine->currvalue>=mine->numparms)
		mine->currvalue-=mine->numparms;
	if (self->statevar)
		*((int *)self->statevar)=mine->currvalue;
	mine->lab->statevar=(void *)mine->optionlist[mine->currvalue];
	mine->lab->childchanged(mine->lab,NULL);
	self->redraw(self,1);
}

int ggiWidgetOptioncyclerEvent(ggi_widget_t self,gii_event *ev,ggi_widget_inputstatus_t status) {

	struct optioncyclerdata *mine=self->priv;
	int is_inside;

	is_inside=(status->mouse.x>=self->pos.x+mine->drawpos.x &&
		   status->mouse.x< self->pos.x+mine->drawpos.x+mine->drawsize.x &&
		   status->mouse.y>=self->pos.y+mine->drawpos.y &&
		   status->mouse.y< self->pos.y+mine->drawpos.y+mine->drawsize.y );

	if (is_inside&&!mine->is_inside) {
		if (self->callback) {
			self->callback(GWT_CB_ENTER,self);
		}
	} else if (!is_inside&&mine->is_inside) {
		if (self->callback) {
			self->callback(GWT_CB_LEAVE,self);
		}
	};
	mine->is_inside=is_inside;

	switch(ev->any.type) {
		case evKeyPress:
		case evKeyRepeat:
			if (ev->key.sym==self->hotkey) {
				next:
				mine->currvalue++;
				update:
				cb_new_value(self);
				if (self->callback) {
					self->callback(GWT_CB_STATECHANGE,self);
				}
				return 1;
			}
			if (self->has_focus) {
				switch(ev->key.sym) {
					case ' ':
					case GIIK_Right:
					case GIIK_Down:
						goto next;
					case GIIK_Left:
					case GIIK_Up:
						prev:
						mine->currvalue--;
						goto update;
				}
			}
			break;
		case evKeyRelease:
			break;
		case evPtrButtonPress:
			if (is_inside) {
				ggiWidgetUnfocus(self);
				self->has_focus=1;
				if (ev->pbutton.button==GII_PBUTTON_PRIMARY) {
					goto next;
				} else if (ev->pbutton.button==GII_PBUTTON_SECONDARY) {
					goto prev;
				}
			}
			break;
		case evPtrButtonRelease:
			break;
	}
	return 0;
}


ggi_widget_t ggiWidgetCreateOptioncycler(va_list ap) {

	ggi_widget_t newid,cline,lab,cycle,frame;
	struct optioncyclerdata *newoptioncycler;
	const char **parms;
	int x;

	static struct ggiWidgetBitmap cyclebm={
		{8,8},"\x7e\xc3\x81\x89\xdb\x7e\x18\x08"
	};

	parms=va_arg(ap,const char **);
	if ( !parms || !parms[0]) return NULL;

	newid=ggiWidgetAllocate();
	if (!newid) return NULL;

	if (NULL==(newoptioncycler=malloc(sizeof(*newoptioncycler)))) {
		free(newid);
		return NULL;
	}
	newoptioncycler->is_inside=0;
	newoptioncycler->currvalue=0;
	
	for(newoptioncycler->numparms=0;parms[newoptioncycler->numparms];newoptioncycler->numparms++);
	newoptioncycler->optionlist=malloc(sizeof(newoptioncycler->optionlist[0])*newoptioncycler->numparms);
	for(x=0;x<newoptioncycler->numparms;x++) {
		newoptioncycler->optionlist[x]=strdup(parms[x]);
	}
	
	newid->priv=newoptioncycler;

	newid->max.x=newid->max.y=-1;

	newid->redraw    =ggiWidgetOptioncyclerRedraw;
	newid->vischanged=ggiWidgetOptioncyclerVischanged;
	newid->childchanged=ggiWidgetOptioncyclerChildchanged;
	newid->process_event=ggiWidgetOptioncyclerEvent;

	newid->getchild   =ggiWidgetSingleChildGet;
	newid->linkchild  =ggiWidgetSingleChildLink;
	newid->unlinkchild=ggiWidgetSingleChildUnlink;

	cycle=ggiWidgetCreateWidget(GWT_BITMAP,&cyclebm);
	lab  =ggiWidgetCreateWidget(GWT_LABEL,NULL);
	newoptioncycler->lab=lab;
	lab->statevar=newoptioncycler->optionlist[0];
	lab->childchanged(lab,NULL);
	cline=ggiWidgetCreateWidget(GWT_CONTAINER_LINE,2,1,lab,cycle,NULL);
	lab->childchanged(lab,NULL);

	frame=ggiWidgetCreateWidget(GWT_FRAME,cline,NULL);

	newid->linkchild(newid,0,frame);
	optioncycler_getsize(newid);

	return newid;
}
